from   ._core import (
    apply, 
    construct_grouper_components
)

class Grouped():
    
    def __init__(self, gids):
        self.inversion_idcs, self.dedup_gids, self.arr_idcs = construct_grouper_components(gids)
        
    def apply(self, fn, *args, **kwargs):
        return apply(self.arr_idcs, fn, *args, **kwargs)


import numpy as np
import pandas as pd

def _split(x, y):
    return np.split(x, y)[:-1]

def _build_arr_idcs(index):
    srt = np.argsort(index, kind="stable")
    uni = np.bincount(index)
    csum = np.cumsum(uni)
    final = _split(srt, csum)
    return final

def apply(arr_idcs, fn, *args, **kwargs):
    return [
        fn( 
            *( a[arr_idc] for a in args ), 
            **{ k: v[arr_idc] for k, v in kwargs.items() }
        )
        for arr_idc in arr_idcs
    ]

def apply_expand(take_idcs, arr_idcs, fn, *args, **kwargs):
    result = np.array(apply(arr_idcs, fn, *args, **kwargs))
    return result[take_idcs]

def construct_grouper_components(gids):
    inversion_idcs, dedup_gids = pd.factorize(gids)
    arr_idcs = _build_arr_idcs(inversion_idcs)
    return inversion_idcs, dedup_gids, arr_idcs


"""
`fastgrouper` is a package for performing fast groupby-apply operations.
"""

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fastgrouper",
    version="0.1.0",
    author="Shreyas Joshi",
    author_email="sjoshistrats@gmail.com",
    description="A package for applying efficient groupby operations.",
    keywords=["fastgrouper", "grouping", "groupby", "fast"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sjoshistrats/fastgrouper",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha"
    ],
    package_dir={"": "python"},
    packages=setuptools.find_packages(where="python"),
    python_requires=">=3.6",
    install_requires=["numpy", "pandas"]
)
